"""
Django middleware integration for Loggier with optimized logging (one log entry per request).
"""
import uuid
import time
import json
import functools
from typing import Dict, List, Any, Optional, Union, Callable, Set

from django.conf import settings
from django.http import HttpRequest, HttpResponse

# Import Loggier client
from ..client import Loggier

# Global Loggier instance for the middleware
_loggier_instance = None

# Registry for decorated views
_tracked_views = set()


def track_endpoint(tags=None, statuses=None, capture_request_body=False, capture_response_body=False):
    """
    Decorator to track specific endpoints.
    
    Args:
        tags: Additional tags for this endpoint
        statuses: Status codes to track (defaults to all)
        capture_request_body: Whether to capture the full request body (careful with sensitive data)
        capture_response_body: Whether to capture the full response body
        
    Example:
        @track_endpoint(tags=['critical', 'payment'])
        def my_view(request):
            ...
    """
    def decorator(view_func):
        # Register this view function for tracking
        _tracked_views.add(view_func)
        
        # Store tracking options on the function
        view_func._loggier_track_options = {
            'tags': tags or [],
            'statuses': statuses or [],
            'capture_request_body': capture_request_body,
            'capture_response_body': capture_response_body
        }
        
        @functools.wraps(view_func)
        def wrapped_view(*args, **kwargs):
            return view_func(*args, **kwargs)
        
        return wrapped_view
    
    return decorator


class LoggierDjangoMiddleware:
    """
    Django middleware for Loggier integration with optimized logging (one log entry per request).
    
    Add to your Django MIDDLEWARE setting:
    
    MIDDLEWARE = [
        ...
        'loggier.integrations.django.LoggierDjangoMiddleware',
        ...
    ]
    
    And configure in settings.py:
    
    LOGGIER = {
        'API_KEY': 'your-api-key',
        'API_URL': 'https://api.loggier.com/api/ingest',
        'ENVIRONMENT': 'development',
        'TAGS': ['django', 'web'],
        'PROJECT_NAME': 'your-project',
        'CAPTURE_REQUEST_DATA': True,
        'LOG_SLOW_REQUESTS': True,
        'SLOW_REQUEST_THRESHOLD': 1.0,  # seconds
        'STATUSES_TO_TRACK': [500, 400, 401, 403],  # Status codes to track
        'CAPTURE_REQUEST_BODY': False,  # Capture request body for tracked statuses
        'CAPTURE_RESPONSE_BODY': True,  # Capture response body for tracked statuses
        'MAX_BODY_SIZE': 16384,  # Max body size to capture (16KB)
        'INCLUDE_STACKTRACE': True  # Include stacktrace for errors
    }
    """
    
    def __init__(self, get_response):
        """
        Initialize the middleware.
        
        Args:
            get_response: The next middleware or view in the chain
        """
        self.get_response = get_response
        
        # Get configuration from Django settings
        config = getattr(settings, "LOGGIER", {})
        
        # Create or get global Loggier instance
        global _loggier_instance
        if _loggier_instance is None:
            _loggier_instance = Loggier(
                api_key=config.get("API_KEY"),
                api_url=config.get("API_URL"),
                project_name=config.get("PROJECT_NAME"),
                environment=config.get("ENVIRONMENT", "development"),
                async_mode=config.get("ASYNC_MODE", True),
                max_batch_size=config.get("MAX_BATCH_SIZE", 100),
                flush_interval=config.get("FLUSH_INTERVAL", 5),
                tags=config.get("TAGS", [])
            )
            
            # Add Django information to global context
            _loggier_instance.context.update_global({
                "framework": "django",
                "django_debug": getattr(settings, "DEBUG", False)
            })
        
        self.logger = _loggier_instance
        
        # Middleware configuration
        self.capture_request_data = config.get("CAPTURE_REQUEST_DATA", True)
        self.log_slow_requests = config.get("LOG_SLOW_REQUESTS", True)
        self.slow_request_threshold = config.get("SLOW_REQUEST_THRESHOLD", 1.0)
        
        # Status codes to track - default to server errors only if not specified
        self.statuses_to_track = set(config.get("STATUSES_TO_TRACK", [500]))
        
        # Body capturing settings
        self.capture_request_body = config.get("CAPTURE_REQUEST_BODY", False)
        self.capture_response_body = config.get("CAPTURE_RESPONSE_BODY", True)
        self.max_body_size = config.get("MAX_BODY_SIZE", 16384)  # 16KB default
        self.include_stacktrace = config.get("INCLUDE_STACKTRACE", True)
    
    def __call__(self, request):
        """
        Process the request and response.
        
        Args:
            request: Django HTTP request
            
        Returns:
            Django HTTP response
        """
        # Check if this is a decorated view that should be tracked
        view_func = None
        is_tracked_view = False
        track_options = {}
        
        # Safely check resolver_match and func
        if hasattr(request, 'resolver_match') and request.resolver_match is not None:
            if hasattr(request.resolver_match, 'func'):
                view_func = request.resolver_match.func
                is_tracked_view = view_func in _tracked_views if view_func else False
                track_options = getattr(view_func, '_loggier_track_options', {}) if is_tracked_view else {}
        
        # Generate request ID and record start time
        request_id = str(uuid.uuid4())
        request.loggier_request_id = request_id
        request.loggier_start_time = time.time()
        
        # Update global context with request ID
        self.logger.context.update_global({"request_id": request_id})
        
        # Determine if we should log this request
        should_log_request = is_tracked_view or self.capture_request_data
        
        # Collect request data if needed
        request_data = None
        if should_log_request:
            # Check if we should capture the request body
            capture_req_body = track_options.get('capture_request_body', self.capture_request_body)
            
            # Collect comprehensive request data
            request_data = self._collect_request_data(request, include_body=capture_req_body)
            
            # Add any decorator tags
            decorator_tags = track_options.get('tags', [])
            if decorator_tags:
                self.logger.context.update_global({"endpoint_tags": decorator_tags})
                
        # Get response (try/except to catch exceptions)
        exception_occurred = False
        exception_info = {}
        
        try:
            response = self.get_response(request)
        except Exception as exc:
            # Process exception
            exception_occurred = True
            
            # Get exception details including full traceback
            import traceback
            exception_info = {
                "type": exc.__class__.__name__,
                "message": str(exc),
            }
            
            if self.include_stacktrace:
                exception_info["traceback"] = "".join(traceback.format_exception(type(exc), exc, exc.__traceback__))
            
            # Re-raise the exception after collecting info
            raise
        finally:
            # Only log the request if we should and if we have timing info
            if should_log_request and hasattr(request, "loggier_start_time"):
                # Calculate request time
                request_time = time.time() - request.loggier_start_time
                
                # Check if we should log due to slow request
                is_slow = self.log_slow_requests and request_time > self.slow_request_threshold
                
                if exception_occurred:
                    # If an exception occurred, always log
                    should_record_log = True
                    log_level = "error"
                    status_code = 500  # Assumed server error
                    
                    # Build log message
                    log_message = f"Exception during request: {request.method} {request.path}"
                    
                    # Start building log context
                    log_context = request_data.copy() if request_data else self._collect_request_data(request)
                    log_context["exception"] = exception_info
                    log_context["request_time"] = request_time
                else:
                    # Get status code from response
                    status_code = response.status_code
                    
                    # Get view-specific statuses to track
                    view_statuses = set(track_options.get('statuses', []))
                    track_statuses = self.statuses_to_track.union(view_statuses)
                    
                    # Determine if this status code should be tracked
                    status_tracked = status_code in track_statuses
                    
                    # Decide if we should log this request based on criteria
                    should_record_log = is_slow or status_tracked or is_tracked_view
                    
                    if should_record_log:
                        # Check if we should capture the response body
                        capture_resp_body = track_options.get('capture_response_body', self.capture_response_body)
                        
                        # Get comprehensive response data
                        resp_data = self._collect_response_data(request, response, request_time, include_body=capture_resp_body)
                        
                        # Build log context
                        if request_data is None:
                            request_data = self._collect_request_data(request)
                        
                        # Combine into a single unified log context
                        log_context = {
                            "request": request_data,
                            "response": resp_data,
                            "request_time": request_time
                        }
                        
                        # Determine log level based on status code
                        if 200 <= status_code < 400:
                            log_level = "info"
                            log_message = f"Request completed: {request.method} {request.path} - {status_code}"
                        elif 400 <= status_code < 500:
                            log_level = "warning"
                            log_message = f"Client error: {request.method} {request.path} - {status_code}"
                        else:  # 500+
                            log_level = "error"
                            log_message = f"Server error: {request.method} {request.path} - {status_code}"
                
                # Log the request with appropriate level if needed
                if should_record_log:
                    if log_level == "info":
                        self.logger.info(log_message, context=log_context)
                    elif log_level == "warning":
                        self.logger.warning(log_message, context=log_context)
                    elif log_level == "error":
                        self.logger.error(log_message, context=log_context, exception=exception_info if exception_occurred else None)
            
            # Clear the global context for the next request
            try:
                # Get the current global context
                global_context = self.logger.context.get_global()
                
                # Clear specific keys in global context
                if global_context and "request_id" in global_context:
                    self.logger.context.update_global({"request_id": None})
                if global_context and "endpoint_tags" in global_context:
                    self.logger.context.update_global({"endpoint_tags": None})
            except TypeError:
                # Handle the case where get_global() requires a key argument
                try:
                    # Try to get and clear request_id
                    request_id = self.logger.context.get_global("request_id")
                    if request_id is not None:
                        self.logger.context.update_global({"request_id": None})
                    
                    # Try to get and clear endpoint_tags
                    endpoint_tags = self.logger.context.get_global("endpoint_tags")
                    if endpoint_tags is not None:
                        self.logger.context.update_global({"endpoint_tags": None})
                except Exception:
                    # If all else fails, just continue without clearing context
                    pass
        
        # Return the response if no exception occurred
        if not exception_occurred:
            return response
    
    def _collect_request_data(self, request, include_body=False):
        """
        Collect comprehensive data from the request object.
        
        Args:
            request: Django HTTP request
            include_body: Whether to include the request body
            
        Returns:
            dict: Request data dictionary
        """
        # Basic request data
        req_data = {
            "method": request.method,
            "path": request.path,
            "full_path": request.get_full_path(),
            "scheme": request.scheme if hasattr(request, 'scheme') else None,
            "remote_addr": request.META.get("REMOTE_ADDR", ""),
            "user_agent": request.META.get("HTTP_USER_AGENT", ""),
            "referer": request.META.get("HTTP_REFERER", ""),
        }
        
        # Add user info if available
        if hasattr(request, "user") and request.user.is_authenticated:
            user_data = {
                "user_id": request.user.id,
                "username": getattr(request.user, 'username', str(request.user)),
                "email": getattr(request.user, 'email', None),
            }
            req_data["user"] = user_data
        
        # Add query parameters if present
        if request.GET:
            req_data["query_params"] = self._filter_sensitive_data(dict(request.GET.items()))
        
        # Add request headers (filtered)
        headers = {}
        for key, value in request.META.items():
            if key.startswith('HTTP_') and not self._is_sensitive_field(key):
                header_name = key[5:].lower().replace('_', '-')
                headers[header_name] = value
        if headers:
            req_data["headers"] = headers
        
        # Add content type and length
        if 'CONTENT_TYPE' in request.META:
            req_data['content_type'] = request.META['CONTENT_TYPE']
        if 'CONTENT_LENGTH' in request.META:
            req_data['content_length'] = request.META['CONTENT_LENGTH']
        
        # Add POST data if appropriate (and not sensitive)
        if include_body and request.method in ['POST', 'PUT', 'PATCH']:
            # Handle different types of request bodies
            body_data = None
            
            # For regular form data
            if hasattr(request, 'POST') and request.POST:
                body_data = self._filter_sensitive_data(dict(request.POST.items()))
            
            # For JSON data
            elif request.content_type == 'application/json' and hasattr(request, 'body'):
                try:
                    json_data = json.loads(request.body.decode('utf-8'))
                    body_data = self._filter_sensitive_data(json_data)
                except Exception:
                    # If we can't parse JSON, include raw body up to max size
                    try:
                        body_data = request.body.decode('utf-8', errors='replace')[:self.max_body_size]
                    except Exception:
                        body_data = "[Unable to decode request body]"
            
            # For other body types, include raw body up to max size if needed
            elif hasattr(request, 'body') and request.body:
                try:
                    body_data = request.body.decode('utf-8', errors='replace')[:self.max_body_size]
                    if len(request.body) > self.max_body_size:
                        body_data += "... [truncated]"
                except Exception:
                    body_data = "[Binary data not shown]"
            
            if body_data is not None:
                req_data["body"] = body_data
        
        # Add view information if available
        if hasattr(request, 'resolver_match') and request.resolver_match:
            view_info = {
                "function": request.resolver_match.view_name,
                "url_name": request.resolver_match.url_name,
                "app_name": request.resolver_match.app_name,
                "namespace": request.resolver_match.namespace,
            }
            
            # Add route pattern if available
            if hasattr(request.resolver_match, 'route'):
                view_info["route"] = request.resolver_match.route
            
            # Add arguments and keyword arguments (filtered)
            if request.resolver_match.args:
                view_info["args"] = request.resolver_match.args
            
            if request.resolver_match.kwargs:
                view_info["kwargs"] = self._filter_sensitive_data(request.resolver_match.kwargs)
            
            req_data["view"] = view_info
        
        # Add files if present
        if hasattr(request, 'FILES') and request.FILES:
            files_info = {}
            for name, file_obj in request.FILES.items():
                files_info[name] = {
                    "name": file_obj.name,
                    "size": file_obj.size,
                    "content_type": file_obj.content_type,
                }
            req_data["files"] = files_info
        
        return req_data
    
    def _collect_response_data(self, request, response, request_time, include_body=False):
        """
        Collect comprehensive data from the response object.
        
        Args:
            request: Django HTTP request
            response: Django HTTP response
            request_time: Request processing time in seconds
            include_body: Whether to include the response body
            
        Returns:
            dict: Response data dictionary
        """
        # Basic response data
        resp_data = {
            "status_code": response.status_code,
            "request_time": round(request_time, 4),
        }
        
        # Add response reason phrase if available
        if hasattr(response, 'reason_phrase') and response.reason_phrase:
            resp_data["reason_phrase"] = response.reason_phrase
        
        # Add headers if available (non-sensitive only)
        if hasattr(response, "headers"):
            headers = {}
            for key, value in response.headers.items():
                if not self._is_sensitive_field(key):
                    headers[key.lower()] = value
            if headers:
                resp_data["headers"] = headers
        
        # Add content type if available
        resp_data["content_type"] = response.get("Content-Type", "")
        
        # Add content length if available
        if hasattr(response, "content"):
            resp_data["content_length"] = len(response.content)
        
        # Add response body for tracked statuses or if explicitly requested
        if include_body and hasattr(response, "content"):
            content_type = response.get("Content-Type", "")
            
            # Handle different content types
            if 'application/json' in content_type:
                try:
                    # Try to parse as JSON
                    body_data = json.loads(response.content.decode('utf-8'))
                    resp_data["body"] = self._filter_sensitive_data(body_data)
                except Exception:
                    # Fallback to raw content
                    resp_data["body"] = response.content.decode('utf-8', errors='replace')[:self.max_body_size]
            elif 'text/' in content_type:
                # Text content
                resp_data["body"] = response.content.decode('utf-8', errors='replace')[:self.max_body_size]
                if len(response.content) > self.max_body_size:
                    resp_data["body"] += "... [truncated]"
            elif include_body:
                # For binary content, just note the size
                resp_data["body"] = f"[Binary content, {len(response.content)} bytes]"
        
        # For error responses, try to parse Django debug info
        if response.status_code >= 400 and hasattr(response, "content"):
            try:
                content = response.content.decode('utf-8', errors='replace')
                
                # Look for Django traceback
                if 'Traceback (most recent call last)' in content:
                    # Extract traceback - simple approach, could be improved
                    start_idx = content.find('Traceback (most recent call last)')
                    if start_idx >= 0:
                        end_idx = content.find('\n\n', start_idx)
                        if end_idx >= 0:
                            traceback = content[start_idx:end_idx].strip()
                            resp_data["traceback"] = traceback
                
                # Look for exception type and value
                if 'Exception Type:' in content and 'Exception Value:' in content:
                    exc_type_start = content.find('Exception Type:') + len('Exception Type:')
                    exc_type_end = content.find('\n', exc_type_start)
                    exc_value_start = content.find('Exception Value:') + len('Exception Value:')
                    exc_value_end = content.find('\n', exc_value_start)
                    
                    if exc_type_end >= 0 and exc_value_end >= 0:
                        exc_type = content[exc_type_start:exc_type_end].strip()
                        exc_value = content[exc_value_start:exc_value_end].strip()
                        
                        resp_data["exception"] = {
                            "type": exc_type,
                            "value": exc_value
                        }
            except Exception:
                # If parsing fails, continue without the extra info
                pass
        
        return resp_data
    
    def _filter_sensitive_data(self, data):
        """
        Filter sensitive data from dictionaries.
        
        Args:
            data: Dictionary to filter
            
        Returns:
            dict: Filtered dictionary
        """
        if not isinstance(data, dict):
            return data
        
        filtered = {}
        for key, value in data.items():
            if self._is_sensitive_field(key):
                filtered[key] = "[FILTERED]"
            elif isinstance(value, dict):
                filtered[key] = self._filter_sensitive_data(value)
            else:
                filtered[key] = value
        return filtered
    
    def _is_sensitive_field(self, field_name):
        """
        Check if a field name contains sensitive information.
        
        Args:
            field_name: Field name to check
            
        Returns:
            bool: True if the field is sensitive
        """
        sensitive_keywords = [
            "password", "token", "auth", "secret", "key", "cookie", 
            "csrf", "session", "card", "credit", "cvv", "ssn", "social",
            "security", "private", "api_key", "apikey", "access_token",
            "refresh_token", "authorization"
        ]
        field_lower = str(field_name).lower()
        return any(keyword in field_lower for keyword in sensitive_keywords)


# Backward compatibility class for legacy applications
class LoggierDjango:
    """
    Legacy Django integration class for backward compatibility.
    For new projects, use LoggierDjangoMiddleware directly.
    """
    
    def __init__(self, **kwargs):
        """Initialize Django integration with a configuration."""
        # Just a compatibility wrapper that returns the middleware
        pass
    
    def get_middleware(self):
        """
        Get a middleware class for use in Django MIDDLEWARE setting.
        Returns a reference to the optimized middleware class.
        """
        return LoggierDjangoMiddleware


import time
import uuid
from typing import Any, Dict, List, Optional, Callable, Union

from fastapi import FastAPI, Request, Response
from fastapi.middleware.base import BaseHTTPMiddleware
from starlette.middleware.base import RequestResponseEndpoint
from starlette.responses import JSONResponse

from ..client import Loggier

class LoggierFastAPI(Loggier):
    """
    FastAPI uygulamaları için Loggier entegrasyonu
    """
    
    def __init__(
        self,
        app: Optional[FastAPI] = None,
        api_key: str = None,
        api_url: str = "https://api.loggier.com/api/ingest",  # host -> api_url olarak değiştirildi
        environment: str = "development",
        async_mode: bool = True,
        max_batch_size: int = 100,  # max_queue_size -> max_batch_size olarak değiştirildi
        flush_interval: int = 5,
        tags: Optional[List[str]] = None,
        context: Optional[Dict[str, Any]] = None,
        log_level: str = "info",  # min_level -> log_level olarak değiştirildi
        capture_request_data: bool = True,
        log_slow_requests: bool = True,
        slow_request_threshold: float = 1.0,  # saniye cinsinden
        exclude_paths: Optional[List[str]] = None
    ):
        """
        FastAPI entegrasyonunu başlat
        
        Args:
            app (FastAPI, optional): FastAPI uygulaması
            api_key (str): Loggier API anahtarı
            api_url (str, optional): API endpoint URL'i (host -> api_url olarak değiştirildi)
            environment (str, optional): Ortam adı
            async_mode (bool, optional): Asenkron log gönderimi yapılsın mı?
            max_batch_size (int, optional): Asenkron modda maksimum kuyruk boyutu
            flush_interval (int, optional): Asenkron modda otomatik gönderim aralığı (saniye)
            tags (List[str], optional): Tüm loglara eklenecek etiketler
            context (Dict[str, Any], optional): Global bağlam bilgileri
            log_level (str, optional): Minimum log seviyesi (min_level -> log_level olarak değiştirildi)
            capture_request_data (bool, optional): HTTP istek verilerini yakala
            log_slow_requests (bool, optional): Yavaş istekleri logla
            slow_request_threshold (float, optional): Yavaş istek eşiği (saniye)
            exclude_paths (List[str], optional): Loglama dışında tutulacak path'ler
        """
        # Ana Loggier sınıfını başlat
        super().__init__(
            api_key=api_key,
            api_url=api_url,  # host -> api_url olarak değiştirildi
            environment=environment,
            async_mode=async_mode,
            max_batch_size=max_batch_size,  # max_queue_size -> max_batch_size olarak değiştirildi
            flush_interval=flush_interval,
            tags=tags,
            context=context,
            log_level=log_level  # min_level -> log_level olarak değiştirildi (eğer varsa)
        )
        
        # FastAPI özellikleri
        self.capture_request_data = capture_request_data
        self.log_slow_requests = log_slow_requests
        self.slow_request_threshold = slow_request_threshold
        self.exclude_paths = exclude_paths or []
        
        # Eğer bir FastAPI uygulaması verilmişse, middleware'i ekle
        if app:
            self.init_app(app)
    
    def init_app(self, app: FastAPI) -> None:
        """
        FastAPI uygulamasına middleware olarak ekle
        
        Args:
            app (FastAPI): FastAPI uygulaması
        """
        app.add_middleware(self.middleware_class)
        
        # Exception handler ekle
        @app.exception_handler(Exception)
        async def exception_handler(request: Request, exc: Exception):
            """
            Yakalanmayan hataları logla
            """
            # Hata bağlamını oluştur
            if hasattr(request.state, "loggier_request_id"):
                with self.context(request_id=request.state.loggier_request_id):
                    await self._capture_exception(request, exc)
            else:
                await self._capture_exception(request, exc)
            
            # Hata yanıtını oluştur
            return JSONResponse(
                status_code=500,
                content={"detail": "Internal Server Error"}
            )
    
    @property
    def middleware_class(self):
        """
        FastAPI middleware sınıfını oluştur
        
        Returns:
            BaseHTTPMiddleware: FastAPI middleware sınıfı
        """
        loggier_instance = self
        
        class LoggierMiddleware(BaseHTTPMiddleware):
            async def dispatch(
                self, request: Request, call_next: RequestResponseEndpoint
            ) -> Response:
                # İstek path'inin hariç tutulanlar arasında olup olmadığını kontrol et
                path = request.url.path
                if any(path.startswith(exclude) for exclude in loggier_instance.exclude_paths):
                    return await call_next(request)
                
                # İstek ID'si oluştur
                request_id = str(uuid.uuid4())
                request.state.loggier_request_id = request_id
                
                # İstek başlangıç zamanını kaydet
                start_time = time.time()
                
                if loggier_instance.capture_request_data:
                    # İstek bağlamını ekle
                    with loggier_instance.context(request_id=request_id):
                        await loggier_instance._capture_request_start(request)
                
                try:
                    # İsteği işle
                    response = await call_next(request)
                    
                    # İstek süresini hesapla
                    request_time = time.time() - start_time
                    
                    if loggier_instance.capture_request_data:
                        # İstek bağlamını ekle
                        with loggier_instance.context(request_id=request_id):
                            await loggier_instance._capture_request_end(request, response, request_time)
                    
                    # Yavaş istek kontrolü
                    if (loggier_instance.log_slow_requests and 
                            request_time > loggier_instance.slow_request_threshold):
                        with loggier_instance.context(request_id=request_id):
                            loggier_instance.warning(
                                f"Yavaş istek tespit edildi: {path} ({request_time:.2f}s)",
                                context={  # extra -> context olarak değiştirildi
                                    "request_time": request_time,
                                    "threshold": loggier_instance.slow_request_threshold,
                                    "path": path,
                                    "method": request.method
                                }
                            )
                    
                    return response
                
                except Exception as exc:
                    # İstek süresini hesapla
                    request_time = time.time() - start_time
                    
                    # Hatayı logla
                    with loggier_instance.context(request_id=request_id):
                        await loggier_instance._capture_exception(request, exc)
                    
                    # Hatayı yeniden fırlat
                    raise exc
        
        return LoggierMiddleware
    
    async def _capture_request_start(self, request: Request) -> None:
        """
        İstek başlangıcını yakala
        
        Args:
            request (Request): FastAPI istek nesnesi
        """
        # İstek bilgilerini topla
        path = request.url.path
        client_host = request.client.host if request.client else "unknown"
        
        req_data = {
            "method": request.method,
            "path": path,
            "remote_addr": client_host,
            "url": str(request.url),
        }
        
        # Query parametrelerini al (hassas veriler hariç)
        if request.query_params:
            filtered_params = {}
            for key, value in request.query_params.items():
                if not self._is_sensitive_data(key):
                    filtered_params[key] = value
            req_data["query_params"] = filtered_params
        
        # HTTP başlıklarını al (hassas veriler hariç)
        if request.headers:
            filtered_headers = {}
            for key, value in request.headers.items():
                if not self._is_sensitive_data(key) and key.lower() not in ["authorization", "cookie"]:
                    filtered_headers[key] = value
            req_data["headers"] = filtered_headers
        
        self.info(f"İstek başladı: {request.method} {path}", context=req_data)  # extra -> context olarak değiştirildi
    
    async def _capture_request_end(self, request: Request, response: Response, request_time: float) -> None:
        """
        İstek sonucunu yakala
        
        Args:
            request (Request): FastAPI istek nesnesi
            response (Response): FastAPI yanıt nesnesi
            request_time (float): İstek süresi (saniye)
        """
        path = request.url.path
        
        # Yanıt bilgilerini topla
        resp_data = {
            "status_code": response.status_code,
            "content_type": response.headers.get("content-type", ""),
            "request_time": round(request_time, 4)
        }
        
        # Başarılı yanıtlar
        if 200 <= response.status_code < 400:
            self.info(
                f"İstek tamamlandı: {request.method} {path} - {response.status_code}",
                context=resp_data  # extra -> context olarak değiştirildi
            )
        # Yönlendirmeler
        elif 300 <= response.status_code < 400:
            self.info(
                f"İstek yönlendirildi: {request.method} {path} - {response.status_code}",
                context=resp_data  # extra -> context olarak değiştirildi
            )
        # İstemci hataları
        elif 400 <= response.status_code < 500:
            self.warning(
                f"İstemci hatası: {request.method} {path} - {response.status_code}",
                context=resp_data  # extra -> context olarak değiştirildi
            )
        # Sunucu hataları
        else:
            self.error(
                f"Sunucu hatası: {request.method} {path} - {response.status_code}",
                context=resp_data  # extra -> context olarak değiştirildi
            )
    
    async def _capture_exception(self, request: Request, exception: Exception) -> None:
        """
        İstek sırasında oluşan hatayı yakala
        
        Args:
            request (Request): FastAPI istek nesnesi
            exception (Exception): Yakalanan hata
        """
        # İstek bilgilerini topla
        path = request.url.path
        client_host = request.client.host if request.client else "unknown"
        
        req_data = {
            "method": request.method,
            "path": path,
            "remote_addr": client_host,
            "url": str(request.url),
        }
        
        # Hatayı logla
        self.exception(
            f"İstek işlenirken hata oluştu: {request.method} {path}",
            exception=exception,  # exception olarak düzeltildi
            context=req_data  # extra -> context olarak değiştirildi
        )
    
    def _is_sensitive_data(self, key: str) -> bool:
        """
        Hassas veri kontrolü
        
        Args:
            key (str): Kontrol edilecek anahtar
        
        Returns:
            bool: Hassas veri ise True
        """
        sensitive_fields = [
            "password", "token", "auth", "secret", "key", "cookie", 
            "csrf", "session", "card", "credit", "cvv", "ssn", "social"
        ]
        key_lower = key.lower()
        return any(sensitive in key_lower for sensitive in sensitive_fields)


# client.py

import os
import sys
import platform
import socket
import uuid
import threading
import traceback
import logging
import json
import time
from typing import Dict, List, Any, Optional, Union, Callable
from datetime import datetime

# Dahili modüller
from .handlers.api_handler import APIHandler
from .handlers.async_handler import AsyncLogHandler
from .handlers.cache_handler import CacheHandler
from .formatters.json_formatter import JSONFormatter
from .utils.context import Context
from .utils.error import ExceptionReporter, capture_exceptions, patch_thread_excepthook
from .utils.network import NetworkMonitor

class Loggier:
    """
    Loggier/Loggier istemci kütüphanesi ana sınıfı.
    Bu sınıf, log kayıtlarını toplar, biçimlendirir ve API'ye gönderir.
    """
    
    DEFAULT_LOG_LEVEL = logging.INFO
    
    # client.py içindeki Loggier sınıfına yapılacak eklemeler

    def __init__(
        self,
        api_key: str,
        project_name: Optional[str] = None,
        environment: str = "development",
        api_url: Optional[str] = None,
        service_name: Optional[str] = None,
        async_mode: bool = True,
        capture_uncaught: bool = True,
        log_level: int = DEFAULT_LOG_LEVEL,
        enable_caching: bool = True,
        cache_dir: Optional[str] = None,
        max_batch_size: int = 20,
        flush_interval: int = 5,
        sensitive_fields: Optional[List[str]] = None,
        max_retries: int = 3,
        http_timeout: int = 5,
        network_check_interval: int = 60,
        tags: Optional[List[str]] = None,
        enable_performance_monitoring: bool = True
    ):
        """
        Loggier istemcisini yapılandırır.
        
        Args:
            api_key: Loggier API anahtarı
            project_name: Proje adı. Belirtilmezse API anahtarına bağlı projeyi kullanır.
            environment: Çalışma ortamı (development, staging, production vb.)
            api_url: API URL adresi. Belirtilmezse varsayılan kullanılır.
            service_name: Servis/uygulama adı. Belirtilmezse otomatik oluşturulur.
            async_mode: True ise loglar arka planda gönderilir, False ise senkron gönderilir.
            capture_uncaught: True ise yakalanmayan istisnalar otomatik raporlanır.
            log_level: Minimum log seviyesi
            enable_caching: Yerel önbellek sistemini etkinleştirir/devre dışı bırakır.
            cache_dir: Önbellek dizini yolu. None ise varsayılan konum kullanılır.
            max_batch_size: Bir defada gönderilecek maksimum log sayısı.
            flush_interval: Saniye cinsinden otomatik boşaltma aralığı.
            sensitive_fields: Maskelenecek hassas alan adları listesi.
            max_retries: Başarısız istekler için maksimum yeniden deneme sayısı.
            http_timeout: HTTP istekleri için zaman aşımı süresi (saniye).
            network_check_interval: Ağ bağlantısı kontrol aralığı (saniye).
            tags: Tüm loglara eklenecek etiketler listesi.
            enable_performance_monitoring: Performans izleme araçlarını etkinleştirir/devre dışı bırakır.
        """
        self.api_key = api_key
        self.project_name = project_name
        self.environment = environment
        self.service_name = service_name or self._generate_service_name()
        self.tags = tags or []
        
        # Logger yapılandırması
        self.logger = logging.getLogger("loggier")
        self.logger.setLevel(log_level)
        
        if not self.logger.handlers:
            handler = logging.StreamHandler()
            formatter = logging.Formatter(
                '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            )
            handler.setFormatter(formatter)
            self.logger.addHandler(handler)
        
        # Bağlam yönetimi
        self.context = Context()
        # Global bağlam oluşturma
        self.context.update_global({
            "environment": environment,
            "service_name": self.service_name,
            "tags": self.tags
        })
        
        # API Handler oluşturma
        api_url = api_url or "https://api.loggier.app"
        self.api_handler = APIHandler(
            api_key=api_key,
            api_url=api_url,
            project_name=project_name,
            max_retries=max_retries,
            timeout=http_timeout
        )
        
        # Biçimlendirici oluşturma
        self.formatter = JSONFormatter(
            sensitive_fields=sensitive_fields or ["password", "token", "api_key", "secret"]
        )
        
        # Ağ durumu izleme
        self.network_monitor = NetworkMonitor(
            check_interval=network_check_interval,
            api_handler=self.api_handler
        )
        
        # Önbellek sistemi
        self.cache_handler = None
        if enable_caching:
            self.cache_handler = CacheHandler(
                cache_dir=cache_dir,
                max_cache_size_mb=100,  # Varsayılan 100MB
                max_cache_age_days=7,  # Varsayılan 7 gün
                use_sqlite=True,
                sync_interval_seconds=60,
                log_level=log_level
            )
            # Ağ durumu değişikliklerini önbelleğe bildir
            self.network_monitor.add_status_callback(self.cache_handler.set_online_status)
        
        # Async handler (arka planda gönderim için)
        self.async_handler = None
        if async_mode:
            self.async_handler = AsyncLogHandler(
                api_handler=self.api_handler,
                max_batch_size=max_batch_size,
                flush_interval=flush_interval,
                cache_handler=self.cache_handler
            )
        
        # Çalışma zamanı bilgisi toplama
        self.runtime_info = self._collect_runtime_info()
        
        # İstisna yakalama
        if capture_uncaught:
            self.exception_reporter = ExceptionReporter(self)
            sys.excepthook = self.exception_reporter.handle_exception
            patch_thread_excepthook()
        
        # Performance Monitor eklendi
        self.performance_monitor = None
        if enable_performance_monitoring:
            from .monitoring.performance import PerformanceMonitor
            self.performance_monitor = PerformanceMonitor(client=self, log_level=log_level)
        
        # İstatistikler
        self.stats = {
            "logs_sent": 0,
            "logs_failed": 0,
            "start_time": datetime.now().isoformat()
        }
        
        self.logger.info(f"Loggier initialized for project '{project_name}' in {environment} environment")


    # Loggier sınıfına eklenecek yeni performans izleme metodları

    def trace_function(
        self,
        name: Optional[str] = None,
        threshold_ms: int = 500,
        tags: Optional[List[str]] = None,
        include_args: bool = False,
        include_return: bool = False,
        log_level: str = "INFO"
    ):
        """
        Fonksiyon çağrılarını izleyen bir dekoratör.
        
        Args:
            name: İzleme için özel isim. None ise fonksiyon adı kullanılır
            threshold_ms: Bu süreden uzun süren çağrılar "yavaş" kabul edilir (milisaniye)
            tags: Logda kullanılacak ek etiketler
            include_args: True ise çağrı argümanları loglanır
            include_return: True ise dönüş değeri loglanır
            log_level: Log seviyesi (INFO, WARNING, vb.)
            
        Returns:
            Callable: Dekorasyon fonksiyonu
            
        Örnek kullanım:
            @loggier.trace_function(threshold_ms=100)
            def slow_function():
                time.sleep(0.2)
        """
        if self.performance_monitor is None:
            # PerformanceMonitor etkin değil, noop dekoratör döndür
            def noop_decorator(func):
                return func
            return noop_decorator
        
        return self.performance_monitor.trace_function(
            name=name,
            threshold_ms=threshold_ms,
            tags=tags,
            include_args=include_args,
            include_return=include_return,
            log_level=log_level
        )

    def trace_http(
        self,
        name: Optional[str] = None,
        threshold_ms: int = 1000,
        tags: Optional[List[str]] = None,
        mask_headers: Optional[List[str]] = None,
        mask_params: Optional[List[str]] = None,
        include_body: bool = False,
        log_level: str = "INFO"
    ):
        """
        HTTP isteklerini izleyen bir dekoratör.
        
        Args:
            name: İzleme için özel isim. None ise fonksiyon adı kullanılır
            threshold_ms: Bu süreden uzun süren istekler "yavaş" kabul edilir (milisaniye)
            tags: Logda kullanılacak ek etiketler
            mask_headers: Maskelenecek HTTP header'ları
            mask_params: Maskelenecek URL parametreleri
            include_body: True ise istek/yanıt gövdeleri loglanır (dikkatli kullanın)
            log_level: Log seviyesi (INFO, WARNING, vb.)
            
        Returns:
            Callable: Dekorasyon fonksiyonu
            
        Örnek kullanım:
            @loggier.trace_http(threshold_ms=2000)
            def fetch_data(url):
                return requests.get(url)
        """
        if self.performance_monitor is None:
            # PerformanceMonitor etkin değil, noop dekoratör döndür
            def noop_decorator(func):
                return func
            return noop_decorator
        
        return self.performance_monitor.trace_http(
            name=name,
            threshold_ms=threshold_ms,
            tags=tags,
            mask_headers=mask_headers,
            mask_params=mask_params,
            include_body=include_body,
            log_level=log_level
        )

    def trace_database(
        self,
        name: Optional[str] = None,
        threshold_ms: int = 100,
        tags: Optional[List[str]] = None,
        include_params: bool = False,
        log_level: str = "INFO"
    ):
        """
        Veritabanı işlemlerini izleyen bir dekoratör.
        
        Args:
            name: İzleme için özel isim. None ise fonksiyon adı kullanılır
            threshold_ms: Bu süreden uzun süren sorgular "yavaş" kabul edilir (milisaniye)
            tags: Logda kullanılacak ek etiketler
            include_params: True ise sorgu parametreleri loglanır (hassas veriler olabilir)
            log_level: Log seviyesi (INFO, WARNING, vb.)
            
        Returns:
            Callable: Dekorasyon fonksiyonu
            
        Örnek kullanım:
            @loggier.trace_database(threshold_ms=50)
            def get_user(user_id):
                return session.query(User).filter_by(id=user_id).first()
        """
        if self.performance_monitor is None:
            # PerformanceMonitor etkin değil, noop dekoratör döndür
            def noop_decorator(func):
                return func
            return noop_decorator
        
        return self.performance_monitor.trace_database(
            name=name,
            threshold_ms=threshold_ms,
            tags=tags,
            include_params=include_params,
            log_level=log_level
        )

    def _generate_service_name(self) -> str:
        """Servis adı belirtilmemişse, çalıştırılan Python dosyasından otomatik bir ad oluşturur."""
        try:
            main_module = sys.modules['__main__']
            if hasattr(main_module, '__file__'):
                return os.path.basename(main_module.__file__).split('.')[0]
            else:
                return f"python-{os.getpid()}"
        except Exception:
            return f"python-{os.getpid()}"
    
    def _collect_runtime_info(self) -> Dict[str, Any]:
        """Sistem ve çalışma zamanı bilgilerini toplar."""
        info = {
            "python_version": platform.python_version(),
            "platform": platform.platform(),
            "system": platform.system(),
            "release": platform.release(),
            "hostname": socket.gethostname(),
            "process_id": os.getpid(),
            "thread_id": threading.get_ident()
        }
        
        # Python paketlerini toplama (sadece ilgili olanları)
        try:
            import pkg_resources
            packages = {pkg.key: pkg.version for pkg in pkg_resources.working_set}
            important_packages = {}
            
            # Önemli paketleri seç
            frameworks = ["django", "flask", "fastapi", "tornado", "bottle", "pyramid", "sanic"]
            databases = ["sqlalchemy", "psycopg2", "pymongo", "redis", "pymysql", "cx_oracle"]
            web = ["requests", "aiohttp", "httpx", "urllib3"]
            
            for pkg_type, pkg_list in [
                ("framework", frameworks), 
                ("database", databases), 
                ("web", web)
            ]:
                for pkg in pkg_list:
                    if pkg in packages:
                        important_packages[pkg] = {
                            "version": packages[pkg],
                            "type": pkg_type
                        }
                        
            info["packages"] = important_packages
        except Exception:
            pass
        
        return info
    
    def _prepare_log_data(
        self,
        level: str,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> Dict[str, Any]:
        """Log verisini hazırlar."""
        # Context birleştirme
        merged_context = self.context.get_context().copy()
        if context:
            merged_context.update(context)
        
        # Etiketleri birleştirme
        merged_tags = self.tags.copy()
        if tags:
            merged_tags.extend(tags)
        
        # Temel log verisi
        log_data = {
            "level": level,
            "message": message,
            "timestamp": datetime.utcnow().isoformat() + "Z",
            "context": merged_context,
            "runtime_info": self.runtime_info,
            "environment": self.environment,
            "service_name": self.service_name,
            "tags": merged_tags
        }
        
        # İstisna bilgisi varsa ekle
        if exception:
            log_data["exception"] = {
                "type": exception.__class__.__name__,
                "message": str(exception),
                "traceback": "".join(traceback.format_exception(
                    type(exception), exception, exception.__traceback__
                ))
            }
        
        return log_data
    
    def _send_log(self, log_data: Dict[str, Any]) -> bool:
        """
        Log verisini gönderir. Async_mode'a göre doğrudan veya arka planda gönderim yapar.
        Bağlantı yoksa veya gönderim başarısız olursa önbelleğe alır.
        
        Args:
            log_data: Gönderilecek log verisi
            
        Returns:
            bool: İşlemin başarı durumu
        """
        # Log verisini biçimlendir
        formatted_data = self.formatter.format(log_data)
        
        # Ağ durumu kontrol et
        is_online = self.network_monitor.is_online()
        
        if not is_online and self.cache_handler:
            # Offline durumda ve önbellek varsa doğrudan önbelleğe al
            success = self.cache_handler.cache_log(formatted_data)
            if success:
                self.logger.debug(f"Log cached: {log_data.get('message', '')[:50]}...")
            else:
                self.logger.warning(f"Failed to cache log: {log_data.get('message', '')[:50]}...")
            return success
        
        if self.async_handler:
            # Async modda kuyruğa ekle
            self.async_handler.enqueue(formatted_data)
            return True
        else:
            # Senkron modda doğrudan gönder
            try:
                self.api_handler.send_log(formatted_data)
                self.stats["logs_sent"] += 1
                return True
            except Exception as e:
                self.logger.warning(f"Failed to send log: {str(e)}")
                self.stats["logs_failed"] += 1
                
                # Gönderim başarısız olursa ve önbellek varsa önbelleğe al
                if self.cache_handler:
                    success = self.cache_handler.cache_log(formatted_data)
                    if success:
                        self.logger.debug(f"Log cached after send failure")
                    else:
                        self.logger.warning(f"Failed to cache log after send failure")
                    return success
                return False
    
    # Log seviyesi metodları
    
    def log(
        self,
        level: str,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> bool:
        """
        Belirtilen seviyede bir log kaydı oluşturur.
        
        Args:
            level: Log seviyesi (DEBUG, INFO, WARNING, ERROR, CRITICAL)
            message: Log mesajı
            exception: İlişkili istisna nesnesi (varsa)
            context: Ek bağlam bilgileri
            tags: Log için özel etiketler
            
        Returns:
            bool: İşlemin başarı durumu
        """
        log_data = self._prepare_log_data(level, message, exception, context, tags)
        return self._send_log(log_data)
    
    def debug(
        self,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> bool:
        """DEBUG seviyesinde log kaydı oluşturur."""
        return self.log("DEBUG", message, exception, context, tags)
    
    def info(
        self,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> bool:
        """INFO seviyesinde log kaydı oluşturur."""
        return self.log("INFO", message, exception, context, tags)
    
    def warning(
        self,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> bool:
        """WARNING seviyesinde log kaydı oluşturur."""
        return self.log("WARNING", message, exception, context, tags)
    
    def error(
        self,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> bool:
        """ERROR seviyesinde log kaydı oluşturur."""
        return self.log("ERROR", message, exception, context, tags)
    
    def critical(
        self,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> bool:
        """CRITICAL seviyesinde log kaydı oluşturur."""
        return self.log("CRITICAL", message, exception, context, tags)
    
    def exception(
        self,
        message: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None
    ) -> bool:
        """
        Mevcut istisnayı yakalar ve ERROR seviyesinde log kaydı oluşturur.
        exception parametresi belirtilmezse, en son yakalanan istisna kullanılır.
        """
        if exception is None:
            exception = sys.exc_info()[1]
            if exception is None:
                raise ValueError("No exception is being handled")
                
        return self.error(message, exception, context, tags)
    
    def context(self, **kwargs) -> Context:
        """
        Geçici bağlam eklemek için bir context manager döndürür.
        
        Örnek kullanım:
            with loggier.context(user_id=123, action="login"):
                loggier.info("User logged in")
        """
        return self.context.push(kwargs)
    
    def flush(self, timeout: Optional[float] = None) -> bool:
        """
        Bekleyen tüm logları hemen gönderir.
        
        Args:
            timeout: Maksimum bekleme süresi (saniye). None ise tüm loglar gönderilene kadar bekler.
            
        Returns:
            bool: Tüm loglar başarıyla gönderildiyse True, değilse False
        """
        if self.async_handler:
            return self.async_handler.flush(timeout)
        return True
    

    # Mevcut get_stats metoduna yapılacak ekleme
    def get_stats(self) -> Dict[str, Any]:
        """
        Loggier istatistiklerini döndürür.
        
        Returns:
            Dict[str, Any]: İstatistik bilgileri
        """
        stats = self.stats.copy()
        
        # Async handler istatistiklerini ekle
        if self.async_handler:
            async_stats = self.async_handler.get_stats()
            stats.update({
                "async_queue_size": async_stats.get("queue_size", 0),
                "async_processed": async_stats.get("processed", 0),
                "async_failed": async_stats.get("failed", 0),
                "async_last_flush": async_stats.get("last_flush", None)
            })
        
        # Önbellek istatistiklerini ekle
        if self.cache_handler:
            cache_stats = self.cache_handler.get_stats()
            stats.update({
                "cache_pending": cache_stats.get("pending_logs", 0),
                "cache_stored": cache_stats.get("stored_logs", 0),
                "cache_synced": cache_stats.get("synced_logs", 0),
                "cache_size": cache_stats.get("cache_size", "0MB"),
                "cache_last_sync": cache_stats.get("last_sync_time", None)
            })
        
        # Ağ durumu bilgisini ekle
        stats["is_online"] = self.network_monitor.is_online()
        stats["network_last_check"] = self.network_monitor.last_check_time
        
        # Performans izleme istatistiklerini ekle
        if self.performance_monitor:
            perf_stats = self.performance_monitor.get_stats()
            stats.update({
                "performance_monitored_calls": perf_stats.get("total_monitored_calls", 0),
                "performance_slow_calls": perf_stats.get("slow_calls", 0),
                "performance_error_calls": perf_stats.get("error_calls", 0)
            })
        
        return stats
  
    def shutdown(self, timeout: Optional[float] = None) -> None:
        """
        Loggier'ı güvenli bir şekilde kapatır ve bekleyen tüm logları göndermeye çalışır.
        
        Args:
            timeout: Maksimum bekleme süresi (saniye). None ise tüm işlemler tamamlanana kadar bekler.
        """
        self.logger.info("Shutting down Loggier")
        
        # Async handler'ı kapat
        if self.async_handler:
            self.async_handler.shutdown(timeout)
        
        # Önbellek handler'ı kapat
        if self.cache_handler:
            self.cache_handler.shutdown()
        
        # Ağ monitörünü kapat
        self.network_monitor.stop()
        
        self.logger.info("Loggier shutdown complete")


# utils/network.py

import logging
import socket
import threading
import time
from typing import List, Callable, Optional, Any
from datetime import datetime

class NetworkMonitor:
    """
    İnternet bağlantısını periyodik olarak kontrol eden ve durum değişikliklerinde
    callback fonksiyonlarını tetikleyen sınıf.
    """
    
    def __init__(
        self,
        check_interval: int = 60,
        api_handler: Optional[Any] = None,
        initial_status: bool = True,
        test_hosts: Optional[List[str]] = None,
        log_level: int = logging.INFO
    ):
        """
        NetworkMonitor sınıfını başlatır.
        
        Args:
            check_interval: Bağlantı kontrol aralığı (saniye)
            api_handler: API bağlantı testleri için kullanılacak APIHandler örneği
            initial_status: Başlangıç bağlantı durumu
            test_hosts: İnternet erişimini test etmek için kullanılacak host listesi
            log_level: Logging seviyesi
        """
        self.logger = logging.getLogger("loggier.network")
        self.logger.setLevel(log_level)
        
        self.check_interval = check_interval
        self.api_handler = api_handler
        self.is_running = False
        self.status_callbacks = []
        self._online = initial_status
        self.last_check_time = datetime.now().isoformat()
        
        # Test edilecek adresler
        self.test_hosts = test_hosts or [
            "8.8.8.8",  # Google DNS
            "1.1.1.1",  # Cloudflare DNS
            "208.67.222.222"  # OpenDNS
        ]
        
        # İzleme thread'ini başlat
        self._start_monitoring()
    
    def _start_monitoring(self):
        """Ağ izleme thread'ini başlatır."""
        if self.is_running:
            return
            
        self.is_running = True
        self.stop_thread = False
        
        self.monitor_thread = threading.Thread(
            target=self._monitor_worker,
            name="LoggierNetworkMonitor",
            daemon=True
        )
        self.monitor_thread.start()
        self.logger.debug("Network monitoring thread started")
    
    def _monitor_worker(self):
        """Periyodik ağ durumu kontrolleri için çalışan thread."""
        while not self.stop_thread:
            try:
                # Ağ durumunu kontrol et
                current_status = self._check_connectivity()
                self.last_check_time = datetime.now().isoformat()
                
                # Durum değişti mi?
                if current_status != self._online:
                    self.logger.info(f"Network status changed: {'Online' if current_status else 'Offline'}")
                    self._online = current_status
                    
                    # Callback'leri tetikle
                    self._notify_status_change(current_status)
            except Exception as e:
                self.logger.error(f"Error in network monitor: {str(e)}")
            
            # Bir sonraki kontrol için bekle
            time.sleep(self.check_interval)
    
    def _check_connectivity(self) -> bool:
        """
        İnternet bağlantısını kontrol eder.
        
        Returns:
            bool: Bağlantı varsa True, yoksa False
        """
        # Önce API handler ile kontrol et (eğer belirtilmişse)
        if self.api_handler:
            try:
                health_check = self.api_handler.check_connectivity()
                if health_check:
                    return True
            except Exception:
                # API kontrol hatası, diğer metodları dene
                pass
        
        # Socket testi - DNS sunucularına bağlantı dene
        for host in self.test_hosts:
            try:
                # 53 = DNS portu, TCP bağlantı kurabilirse internet vardır
                socket.create_connection((host, 53), timeout=3.0)
                return True
            except OSError:
                continue
        
        # HTTP isteği ile kontrol et
        try:
            import urllib.request
            urllib.request.urlopen('http://www.google.com', timeout=3)
            return True
        except Exception:
            pass
        
        # Tüm kontroller başarısız olduysa offline kabul et
        return False
    
    def add_status_callback(self, callback: Callable[[bool], None]) -> None:
        """
        Ağ durum değişikliğini izlemek için callback fonksiyonu ekler.
        
        Args:
            callback: Durum değiştiğinde çağrılacak fonksiyon.
                     Fonksiyon bir bool parametresi alır (True=Online, False=Offline)
        """
        if callback not in self.status_callbacks:
            self.status_callbacks.append(callback)
    
    def remove_status_callback(self, callback: Callable[[bool], None]) -> None:
        """Durum değişikliği callback'ini kaldırır."""
        if callback in self.status_callbacks:
            self.status_callbacks.remove(callback)
    
    def _notify_status_change(self, status: bool) -> None:
        """Tüm kayıtlı callback'lere durum değişikliğini bildirir."""
        for callback in self.status_callbacks:
            try:
                callback(status)
            except Exception as e:
                self.logger.error(f"Error in network status callback: {str(e)}")
    
    def is_online(self) -> bool:
        """
        Mevcut ağ durumunu döndürür.
        
        Returns:
            bool: Internet bağlantısı varsa True, yoksa False
        """
        return self._online
    
    def check_now(self) -> bool:
        """
        Ağ durumunu hemen kontrol eder ve sonucu döndürür.
        
        Returns:
            bool: Internet bağlantısı varsa True, yoksa False
        """
        try:
            current_status = self._check_connectivity()
            self.last_check_time = datetime.now().isoformat()
            
            # Durum değiştiyse callback'leri tetikle
            if current_status != self._online:
                self.logger.info(f"Network status changed: {'Online' if current_status else 'Offline'}")
                self._online = current_status
                self._notify_status_change(current_status)
            
            return current_status
        except Exception as e:
            self.logger.error(f"Error checking network connectivity: {str(e)}")
            return self._online  # Hata durumunda mevcut durumu koruyoruz
    
    def stop(self) -> None:
        """İzleme thread'ini durdurur."""
        self.logger.debug("Stopping network monitor")
        self.stop_thread = True
        
        # Thread'in sonlanmasını bekle
        if hasattr(self, 'monitor_thread') and self.monitor_thread.is_alive():
            self.monitor_thread.join(timeout=2.0)
            
        self.is_running = False


import json
import datetime
import traceback
import platform
import socket
import os
import sys
from typing import Any, Dict, List, Optional, Union

class JSONFormatter:
    """
    Log kayıtlarını JSON formatına dönüştüren formatter
    """
    
    def __init__(
        self,
        include_runtime_info: bool = True,
        include_timestamp: bool = True,
        include_hostname: bool = True,
        include_process_info: bool = True,
        max_traceback_length: int = 4096,
        max_message_length: int = 1024,
        sensitive_fields: Optional[List[str]] = None
    ):
        """
        JSONFormatter'ı başlat
        
        Args:
            include_runtime_info (bool, optional): Çalışma zamanı bilgilerini ekle
            include_timestamp (bool, optional): Zaman damgasını ekle
            include_hostname (bool, optional): Sunucu adını ekle
            include_process_info (bool, optional): Proses bilgilerini ekle
            max_traceback_length (int, optional): Maksimum traceback uzunluğu
            max_message_length (int, optional): Maksimum mesaj uzunluğu
            sensitive_fields (List[str], optional): Hassas veri içeren anahtar listesi
        """
        self.include_runtime_info = include_runtime_info
        self.include_timestamp = include_timestamp
        self.include_hostname = include_hostname
        self.include_process_info = include_process_info
        self.max_traceback_length = max_traceback_length
        self.max_message_length = max_message_length
        self.sensitive_fields = sensitive_fields or [
            "password", "token", "auth", "secret", "key", "cookie", 
            "csrf", "session", "card", "credit", "cvv", "ssn", "social"
        ]
    
    def format(self, log_data: Dict[str, Any]) -> Dict[str, Any]:
        """
        Log verisini JSON formatına dönüştür
        
        Args:
            log_data (Dict[str, Any]): Format edilecek log verisi
            
        Returns:
            Dict[str, Any]: JSON formatında log verisi
        """
        # Parametre adlarını düzeltmek için burayı yeniden yazıyoruz
        message = log_data.get('message', '')
        level = log_data.get('level', 'INFO')
        exception = log_data.get('exception')
        context = log_data.get('context', {})
        tags = log_data.get('tags', [])
        environment = log_data.get('environment', 'development')
        
        return self._format_log_data(message, level, exception, context, tags, environment)
    
    def _format_log_data(
        self,
        message: str,
        level: str,
        exception: Optional[Exception] = None,
        context: Optional[Dict[str, Any]] = None,
        tags: Optional[List[str]] = None,
        environment: str = "development"
    ) -> Dict[str, Any]:
        """
        Log kaydını JSON formatına dönüştür
        
        Args:
            message (str): Log mesajı
            level (str): Log seviyesi (info, warning, error, critical)
            exception (Exception, optional): İstisna nesnesi
            context (Dict[str, Any], optional): Bağlam bilgileri
            tags (List[str], optional): Etiketler
            environment (str, optional): Ortam adı
        
        Returns:
            Dict[str, Any]: JSON formatında log verisi
        """
        # Temel log verisini oluştur
        log_data = {
            "message": self._truncate(message, self.max_message_length),
            "level": level,
            "environment": environment
        }
        
        # Zaman damgası ekle
        if self.include_timestamp:
            log_data["timestamp"] = datetime.datetime.utcnow().isoformat()
        
        # Hassas verileri temizle ve metadata ekle
        if context:
            log_data["context"] = self._sanitize_sensitive_data(context)
        
        # Etiketleri ekle
        if tags:
            log_data["tags"] = tags
        
        # İstisna bilgilerini ekle
        if exception:
            log_data["exception"] = self._format_exception(exception)
        
        # Çalışma zamanı bilgilerini ekle
        if self.include_runtime_info:
            log_data["runtime_context"] = self._get_runtime_info()
        
        return log_data
    
    def _format_exception(self, exception: Exception) -> Dict[str, Any]:
        """
        İstisna nesnesini formatla
        
        Args:
            exception (Exception): İstisna nesnesi
        
        Returns:
            Dict[str, Any]: Formatlanmış istisna verisi
        """
        exc_type = type(exception)
        
        exception_data = {
            "type": exc_type.__name__,
            "message": str(exception),
            "args": [str(arg) for arg in getattr(exception, 'args', [])],
        }
        
        if hasattr(exception, "__traceback__") and exception.__traceback__:
            tb_text = ''.join(traceback.format_exception(
                exc_type, exception, exception.__traceback__
            ))
            exception_data["traceback"] = self._truncate(tb_text, self.max_traceback_length)
        else:
            tb_text = traceback.format_exc()
            if tb_text and tb_text.strip() != "NoneType: None":
                exception_data["traceback"] = self._truncate(tb_text, self.max_traceback_length)
        
        return exception_data
    
    def _get_runtime_info(self) -> Dict[str, Any]:
        """
        Çalışma zamanı bilgilerini topla
        
        Returns:
            Dict[str, Any]: Çalışma zamanı bilgileri
        """
        info = {
            "python_version": platform.python_version(),
            "platform": platform.platform(),
            "system": platform.system(),
            "python_implementation": platform.python_implementation(),
        }
        
        if self.include_hostname:
            try:
                info["hostname"] = socket.gethostname()
                info["ip"] = socket.gethostbyname(socket.gethostname())
            except Exception:
                pass
        
        if self.include_process_info:
            info["process_id"] = os.getpid()
            info["thread_id"] = os.getpid()  # Gerçek thread ID'sini almak için threading modülü kullanılabilir
        
        # Uygulama bilgilerini topla
        try:
            main_module = sys.modules['__main__']
            main_file = getattr(main_module, '__file__', None)
            if main_file:
                info["main_file"] = os.path.basename(main_file)
        except Exception:
            pass
        
        return info
    
    def _sanitize_sensitive_data(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """
        Hassas verileri maskele
        
        Args:
            data (Dict[str, Any]): Temizlenecek veri
        
        Returns:
            Dict[str, Any]: Temizlenmiş veri
        """
        if not data:
            return data
        
        sanitized = {}
        
        for key, value in data.items():
            # Anahtarın hassas olup olmadığını kontrol et
            key_lower = key.lower()
            is_sensitive = any(sensitive in key_lower for sensitive in self.sensitive_fields)
            
            if is_sensitive and isinstance(value, (str, int, float, bool)):
                # Hassas veriyi maskele
                sanitized[key] = "********"
            elif isinstance(value, dict):
                # İç içe sözlükleri temizle
                sanitized[key] = self._sanitize_sensitive_data(value)
            elif isinstance(value, list) and all(isinstance(item, dict) for item in value):
                # Sözlük listelerini temizle
                sanitized[key] = [self._sanitize_sensitive_data(item) for item in value]
            else:
                # Normal verileri aynen kopyala
                sanitized[key] = value
        
        return sanitized
    
    def _truncate(self, text: str, max_length: int) -> str:
        """
        Metni belirli bir uzunlukta kısalt
        
        Args:
            text (str): Kısaltılacak metin
            max_length (int): Maksimum uzunluk
        
        Returns:
            str: Kısaltılmış metin
        """
        if len(text) <= max_length:
            return text
        
        # Metni kısalt ve sonuna "..." ekle
        return text[:max_length - 3] + "..."
    
    def to_json_string(self, log_data: Dict[str, Any]) -> str:
        """
        Log verisini JSON string'e dönüştür
        
        Args:
            log_data (Dict[str, Any]): Log verisi
        
        Returns:
            str: JSON formatında string
        """
        class DateTimeEncoder(json.JSONEncoder):
            """JSON encoder that handles datetime objects."""
            def default(self, obj):
                if isinstance(obj, datetime.datetime):
                    return obj.isoformat()
                return super().default(obj)
        
        return json.dumps(log_data, cls=DateTimeEncoder, ensure_ascii=False)


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="loggier",
    version="0.1.7",
    author="Yigit ALTINTAS",
    author_email="'yigit.altintas@loggier.app",
    description="Python tabanlı backend uygulamaları için log yönetim kütüphanesi",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yigitaltintas/loggier",
    project_urls={
        "Bug Tracker": "https://github.com/yigitaltintas/loggier/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: System :: Logging",
    ],
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=[
        "requests>=2.25.0",
        "tqdm>=4.61.0",
        "psutil>=5.8.0",
    ],
    keywords="logging, error tracking, monitoring, backend, api",
)


import sys
import traceback
from functools import wraps
from typing import Any, Callable, Dict, Optional, Type, TypeVar, cast

# Client referansını Type Hinting için tanımla
LoggierType = TypeVar('LoggierType')


def capture_exceptions(
    logger: LoggierType,
    level: str = "error",
    reraise: bool = True,
    exc_types: Optional[Type[Exception]] = None,
    message: str = "Fonksiyon çalışırken bir hata oluştu",
    context: Optional[Dict[str, Any]] = None
):
    """
    Fonksiyon çalışırken oluşan hataları otomatik olarak yakala ve logla

    Args:
        logger (Loggier): Loggier instance
        level (str, optional): Log seviyesi. Defaults to "error".
        reraise (bool, optional): Hatayı yeniden fırlat. Defaults to True.
        exc_types (Type[Exception], optional): Yakalanacak hata tipleri. Defaults to None (tüm hatalar).
        message (str, optional): Log mesajı. Defaults to "Fonksiyon çalışırken bir hata oluştu".
        context (Dict[str, Any], optional): Ek bağlam bilgileri. Defaults to None.

    Returns:
        Callable: Decorator fonksiyonu

    Example:
        @capture_exceptions(logger, level="error", message="Kullanıcı işlemi başarısız")
        def process_user(user_id):
            # ...
    """

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                # Belirli hata tipleri belirtilmişse kontrol et
                if exc_types and not isinstance(e, exc_types):
                    raise

                # Fonksiyon bilgilerini topla
                func_info = {
                    "function": func.__name__,
                    "module": func.__module__,
                    "args": str(args)[:200],  # Çok uzun olmasın diye kısıtla
                    "kwargs": str({k: str(v)[:50] for k, v in kwargs.items()})  # Hassas veriler için kısıtla
                }

                # Bağlam bilgilerini birleştir
                combined_context = {**func_info, **(context or {})}

                # Traceback bilgisini ekle
                combined_context["traceback_details"] = {
                    "exception_type": type(e).__name__,
                    "exception_message": str(e),
                    "traceback": traceback.format_exc()
                }

                # Hatayı logla
                if level == "critical":
                    logger.critical(message, exception=e, context=combined_context)
                elif level == "error":
                    logger.error(message, exception=e, context=combined_context)
                elif level == "warning":
                    logger.warning(message, exception=e, context=combined_context)
                else:
                    logger.error(message, exception=e, context=combined_context)

                # Yeniden fırlat
                if reraise:
                    raise

                return None

        return wrapper

    return decorator


class ExceptionReporter:
    """
    Yakalanmayan (uncaught) istisnaları raporlamak için sınıf
    """

    def __init__(self, logger: LoggierType):
        """
        ExceptionReporter'ı başlat

        Args:
            logger (Loggier): Loggier instance
        """
        self.logger = logger
        self.original_excepthook = sys.excepthook

    def handle_exception(self, exc_type, exc_value, exc_traceback):
        """
        Yakalanmayan istisnaları için excepthook metodu
        """
        # Tam traceback bilgisini al
        try:
            # Formatlanmış traceback'i al
            tb_lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
            formatted_traceback = ''.join(tb_lines)

            # Exception detaylarını topla
            exception_details = {
                "exception_type": exc_type.__name__ if exc_type else "Unknown",
                "exception_message": str(exc_value) if exc_value else "No message",
                "exception_module": getattr(exc_type, '__module__', 'Unknown') if exc_type else "Unknown",
                "traceback": formatted_traceback,
                "traceback_summary": traceback.format_exception_only(exc_type, exc_value)
            }

            # Eğer traceback varsa, son frame bilgisini de ekle
            if exc_traceback:
                last_frame = exc_traceback
                while last_frame.tb_next:
                    last_frame = last_frame.tb_next

                frame_info = last_frame.tb_frame
                exception_details["last_frame"] = {
                    "filename": frame_info.f_code.co_filename,
                    "function": frame_info.f_code.co_name,
                    "line_number": last_frame.tb_lineno,
                    "local_vars": {k: str(v)[:100] for k, v in frame_info.f_locals.items() if not k.startswith('_')}
                }

        except Exception as format_error:
            # Traceback formatlarken hata olursa basit bilgi ver
            exception_details = {
                "exception_type": str(exc_type) if exc_type else "Unknown",
                "exception_message": str(exc_value) if exc_value else "No message",
                "format_error": str(format_error),
                "traceback": "Could not format traceback"
            }

        # Hatayı logla
        self.logger.critical(
            "Yakalanmayan bir istisna oluştu",
            exception=exc_value,
            context=exception_details
        )

        # Orijinal excepthook'u çağır
        self.original_excepthook(exc_type, exc_value, exc_traceback)

    # Geriye dönük uyumluluk için excepthook metod adını da kullanılabilir tut
    excepthook = handle_exception

    def enable(self):
        """
        Yakalanmayan istisna raporlamayı etkinleştir
        """
        sys.excepthook = self.handle_exception

    def disable(self):
        """
        Yakalanmayan istisna raporlamayı devre dışı bırak
        """
        sys.excepthook = self.original_excepthook


def patch_thread_excepthook():
    """
    Thread'lerde yakalanmayan istisnaları raporlamak için threading modülünü yama
    """
    import threading

    # Orijinal thread run metodunu kaydet
    old_thread_run = threading.Thread.run

    def new_thread_run(self, *args, **kwargs):
        try:
            old_thread_run(self, *args, **kwargs)
        except Exception:
            # Thread excepthook'u çağır (sys.excepthook veya özel excepthook)
            exc_info = sys.exc_info()
            if exc_info[0] is not None:
                sys.excepthook(*exc_info)

    # Thread sınıfının run metodunu değiştir
    threading.Thread.run = new_thread_run


def format_exception_for_logging(exception: Exception) -> Dict[str, Any]:
    """
    Exception'ı loglama için detaylı formatlama yapar

    Args:
        exception: Formatlanacak exception

    Returns:
        Dict: Detaylı exception bilgisi
    """
    try:
        exc_type = type(exception)
        exc_value = exception
        exc_traceback = exception.__traceback__

        # Temel exception bilgileri
        exception_data = {
            "type": exc_type.__name__,
            "message": str(exc_value),
            "module": getattr(exc_type, '__module__', 'Unknown'),
            "args": [str(arg) for arg in getattr(exception, 'args', [])]
        }

        # Traceback bilgisi
        if exc_traceback:
            # Formatlanmış traceback
            tb_lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
            exception_data["traceback"] = ''.join(tb_lines)

            # Traceback özeti
            exception_data["traceback_summary"] = traceback.format_exception_only(exc_type, exc_value)

            # Frame bilgileri
            frames = []
            tb = exc_traceback
            while tb:
                frame = tb.tb_frame
                frames.append({
                    "filename": frame.f_code.co_filename,
                    "function": frame.f_code.co_name,
                    "line_number": tb.tb_lineno,
                    "code_context": linecache.getline(frame.f_code.co_filename, tb.tb_lineno).strip()
                })
                tb = tb.tb_next

            exception_data["frames"] = frames

            # Son frame'in local değişkenleri (güvenli şekilde)
            if frames:
                last_frame = exc_traceback
                while last_frame.tb_next:
                    last_frame = last_frame.tb_next

                try:
                    local_vars = {}
                    for k, v in last_frame.tb_frame.f_locals.items():
                        if not k.startswith('_'):
                            try:
                                # Güvenli string dönüşümü
                                local_vars[k] = str(v)[:100]
                            except:
                                local_vars[k] = "<could not serialize>"

                    exception_data["local_variables"] = local_vars
                except:
                    exception_data["local_variables"] = "Could not extract local variables"

        # Eğer exception'ın özel attributeleri varsa
        if hasattr(exception, '__dict__'):
            custom_attrs = {}
            for key, value in exception.__dict__.items():
                if not key.startswith('_'):
                    try:
                        custom_attrs[key] = str(value)[:100]
                    except:
                        custom_attrs[key] = "<could not serialize>"

            if custom_attrs:
                exception_data["custom_attributes"] = custom_attrs

        return exception_data

    except Exception as format_error:
        # Formatlarken hata olursa minimal bilgi döndür
        return {
            "type": str(type(exception).__name__),
            "message": str(exception),
            "format_error": str(format_error),
            "traceback": "Could not format exception details"
        }


# Linecache import ekle
import linecache